import * as React from 'react';
export declare type Omit<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;
/**
 * Optional props for styleable components. If these props are present, they will automatically be
 * used by Foundation when applying theming and styling.
 */
export interface IStyleableComponentProps<TViewProps, TStyleSet, TTheme, TScheme> {
    styles?: IStylesProp<TViewProps, TStyleSet, TTheme>;
    theme?: TTheme;
}
/**
 * Props added by Foundation for styles functions.
 */
export interface IStyledProps<TTheme> {
    theme: TTheme;
}
/**
 * Styles functions that take in view props and foundation supplied properties for processing.
 */
export declare type IStylesFunction<TViewProps, TStyleSet, TTheme> = (props: TViewProps & IStyledProps<TTheme>) => Partial<TStyleSet>;
/**
 * Styles can be a function or an object taking in TViewProps for processing.
 */
export declare type IStylesProp<TViewProps, TStyleSet, TTheme> = IStylesFunction<TViewProps, TStyleSet, TTheme> | Partial<TStyleSet>;
/**
 * Enforce props contract on state components, including the view prop and its shape.
 */
export declare type IStateComponentProps<TComponentProps, TViewProps> = TComponentProps & {
    renderView: React.StatelessComponent<TViewProps>;
};
/**
 * Imposed state component props contract with styling props as well as a renderView
 * prop that the StateComponent should make use of in its render output (and should be its only render output.)
 */
export declare type IStateComponentType<TComponentProps, TViewProps> = React.ComponentType<IStateComponentProps<TComponentProps, TViewProps>>;
/**
 * The props that get passed to view components.
 */
export declare type IViewComponentProps<TViewProps, TProcessedStyleSet> = TViewProps & {
    classNames: TProcessedStyleSet;
};
/**
 * A helper type for defining view components, including its properties.
 */
export declare type IViewComponent<TViewProps, TProcessedStyleSet> = React.StatelessComponent<IViewComponentProps<TViewProps, TProcessedStyleSet>>;
/**
 * Component used by foundation to tie elements together.
 * @see createComponent for generic type documentation.
 */
export interface IComponentOptions<TComponentProps, TViewProps, TStyleSet, TProcessedStyleSet, TTheme, TStatics> {
    /**
     * Display name to identify component in React hierarchy.
     */
    displayName: string;
    /**
     * List of fields which can be customized.
     */
    fields?: string[];
    /**
     * Styles prop to pass into component.
     */
    styles: IStylesProp<TViewProps, TStyleSet, TTheme>;
    /**
     * React view stateless component.
     */
    view: IViewComponent<TViewProps, TProcessedStyleSet>;
    /**
     * Optional state component that processes TComponentProps into TViewProps.
     */
    state?: IStateComponentType<TComponentProps, TViewProps>;
    /**
     * Optional static object to assign to constructed component.
     */
    statics?: TStatics;
}
/**
 * Variant of IComponentOptions for stateful components with appropriate typing and required properties.
 */
export declare type IComponent<TComponentProps, TViewProps, TStyleSet, TProcessedStyleSet, TTheme, TStatics> = IComponentOptions<TComponentProps, TViewProps, TStyleSet, TProcessedStyleSet, TTheme, TStatics> & Required<Pick<IComponentOptions<TComponentProps, TComponentProps, TStyleSet, TProcessedStyleSet, TTheme, TStatics>, 'state'>>;
/**
 * Variant of IComponentOptions for stateless components with appropriate typing and required properties.
 */
export declare type IStatelessComponent<TComponentProps, TStyleSet, TProcessedStyleSet, TTheme, TStatics> = Omit<IComponentOptions<TComponentProps, TComponentProps, TStyleSet, TProcessedStyleSet, TTheme, TStatics>, 'state'>;
/**
 * Providers used by createComponent to process and apply styling.
 */
export interface IComponentProviders<TViewProps, TStyleSet, TProcessedStyleSet, TContext, TTheme, TScheme> {
    /**
     * A required provider that merges multiple TStyleSets to create a TProcessedStyleSet that will be passed onto views components.
     */
    mergeStyleSets: (...styles: (Partial<TStyleSet> | undefined)[]) => TProcessedStyleSet;
    /**
     * A required provider for accessing global customizations as a fallback for contextual customizations.
     * @param {string} scope Name of scope for targeted customizations.
     * @param {TContext} context Current context including any contextual customizations.
     * @param {string[]} fields Optional list of props that can be customized.
     */
    getCustomizations: (scope: string, context: TContext, fields?: string[]) => IStyleableComponentProps<TViewProps, TStyleSet, TTheme, TScheme>;
    /**
     * React context provider based on TContext.
     */
    CustomizerContext: React.Context<TContext>;
}
/**
 * Assembles a higher order component based on the following: styles, theme, view, and state.
 * Imposes a separation of concern and centralizes styling processing to increase ease of use and robustness
 * in how components use and apply styling and theming.
 *
 * Automatically merges and applies themes and styles with theme / styleprops having the highest priority.
 * State component, if provided, is passed in props for processing. Props from state / user are automatically processed
 * and styled before finally being passed to view.
 *
 * State components should contain all stateful behavior and should not generate any JSX, but rather simply call the view prop.
 * Views should simply be stateless pure functions that receive all props needed for rendering their output.
 * State component is optional. If state not provided, created component is essentially a functional stateless component.
 *
 * TComponentProps: A styleable props interface for the created component.
 * TViewProps: The props specific to the view, including processed properties outputted by optional state component. If state
 * component is not provided, TComponentProps is the same as TViewProps.
 * TStyleSet: The type for styles properties.
 * TProcessedStyleSet: The type provided by mergeStyleSets provider after processing TStyleSet and provided to views.
 * TTheme: The type for theme properties as well as the getTheme provider.
 * TScheme: The type for identifying schemes.
 * TStatics: Static type for statics applied to created component object.
 *
 * @param {IComponent} component
 * @param {IComponentProviders} providers
 */
export declare function createComponent<TComponentProps, TViewProps, TStyleSet, TProcessedStyleSet, TContext, TTheme, TScheme, TStatics>(component: IComponent<TComponentProps, TViewProps, TStyleSet, TProcessedStyleSet, TTheme, TStatics>, providers: IComponentProviders<TViewProps, TStyleSet, TProcessedStyleSet, TContext, TTheme, TScheme>): React.StatelessComponent<TComponentProps> & TStatics;
/**
 * A wrapper function around createComponent to confine generics and component properties for stateless components.
 *
 * @see {@link createComponent} for more information.
 */
export declare function createStatelessComponent<TComponentProps, TStyleSet, TProcessedStyleSet, TContext, TTheme, TScheme, TStatics>(component: IStatelessComponent<TComponentProps, TStyleSet, TProcessedStyleSet, TTheme, TStatics>, providers: IComponentProviders<TComponentProps, TStyleSet, TProcessedStyleSet, TContext, TTheme, TScheme>): React.StatelessComponent<TComponentProps> & TStatics;
